/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 5.2 (Ahrens, immediate acceptance), p. 116  
 *
 * this file: algo05_02.c
 *
 * For a demo how to use this algorithm see the file "demo_algo05_02.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

 static double *b=NULL, *fb=NULL, *a=NULL, *nu=NULL, atotal, (*pdfh)(double);


int setup05_02( double (*pdf)(double), 
                int N, double constructionp[])
/*
 * Algorithm 5.2 (Ahrens, immediate acceptance), p. 116  
 *
 * pdf... function pointer to the density
 * N  ... is the number of construction points (in the book this is N+1)
 * constructionp[] ... array holding the construction points
 * constructionp[0] is the left, constructionp[N-1] the right border of the domain
 *
 */
{    int i;  
     if(b!=NULL){
       free(b);
       free(fb);
       free(a);
       free(nu);
     }
     pdfh =pdf;
     b = malloc((N+1)*sizeof(double));  
     fb = malloc((N+1)*sizeof(double));  
     a = malloc((N+1)*sizeof(double));  
     nu = malloc((N+1)*sizeof(double));  


     for(i=0;i<=N;i++){ 
       b[i] = constructionp[i];
/*1*/  fb[i] = (*pdf)(b[i]);
     }
     atotal = 0.;
     for(i=0;i<=N;i++){
/*2*/  nu[i] = (*pdfh)(b[i])/(*pdfh)(b[i-1]);
/*3*/  a[i] = fb[i-1]*(b[i]-b[i-1]);
/*4*/  atotal += a[i];
     }
     printf("Setup:\n   construction points and intervals\n");
     for(i=1;i<N;i++){
       printf("   [%d] <%f;%f> area %f nu %f \n",i,b[i-1],b[i],a[i],nu[i]);
     }     
     printf("   total area below hat: %f\n\n",atotal);
     return 1;
}

double generator05_02()
/* is called after the setup to generator reandom variates */
{  double v,y,x,asum;
   int j;  

   if(b==NULL){
	printf("ERROR generator05_02: you have to run the setup first!!\n");
	return -9999.;
      }
/*5*/ while(1){
/*6*/   v = atotal*uniform();
/*7*/   j = 1;
	asum = a[1];
	while(v > asum){
	  j++;
	  asum += a[j];
	}
/*8*/   v = (v - (asum-a[j]))/a[j];
/*9*/   if(v <=nu[j]){
/*10*/    x = b[j-1] + (v/nu[j])*(b[j]-b[j-1]);
/*11*/    return x;
        }
/*12*/  else{
/*13*/    x = b[j-1] + (v-nu[j])/(1.-nu[j])*(b[j]-b[j-1]);
/*14*/    y = fb[j]+(fb[j-1] - fb[j])*uniform();
/*15*/    if(y <= (*pdfh)(x)) 
/*16*/      return x;
        }
   }
}

int free05_02()
/* frees the allocated memory */
{
      free(b);
      b = NULL;
      free(fb);
      fb = NULL;
      free(a);
      a = NULL;
      free(nu);
      nu = NULL;
      return 1;
}


