/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 6.3 (Monotone-unbounded, p. 130)
 *
 * this file: algo06_03.c
 *
 * For a demo how to use this algorithm see the file "demo_algo06_03.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

static double f0 = 0., ah=0., a2, r, mur, k, (*pdfh)(double);

int setup06_03( double (*pdf)(double), double ri, double muri)
/*
 * Algorithm 6.3 (Monotone-unbounded, p. 130)
 *
 * pdf ... function pointer to the density, which must be monotone on (0,infinity)
 *         (the pdf must be a real density, i.e. Af = 1.)
 * ri ... the r-th moment of the distribution has a known upper bound
 * muri ... upper bound for teh r-th moment
 */
{     double a3;
      pdfh = pdf;
      r = ri;
      mur = muri;
/*1*/ f0 = (*pdf)(0.);
      k = (r+1.)*mur;
/*2*/ a2=log(f0)+log((r+1.)*mur)/r;
      a3=1./r;
/*3*/ ah = 1. + a2 + a3;
      printf("Setup06_03:\n a2 %f ; total area below hat: %f\n\n",a2,ah);
      return 1;
} 

double generator06_03()
/* is called after the setup to generate random variates */
{ double u,v,x,h;

/*4*/  while(1){
/*5*/    u = ah*uniform();
         v = uniform();
/*6*/    if(u <= 1.){
/*7*/      x=u/f0;
           h = f0;
         }
/*8*/    else{
/*9*/      if(u <= 1+a2){
/*10*/       x = exp(u-1.-log(f0));
             h = 1./x;
           }
/*11*/     else{
/*12*/       x = exp(log(k/(1-r*(u-1-a2)))/r);
             h = k*exp(-(r+1.)*log(x));
           }
         }
/*13*/   if(v*h<=(*pdfh)(x))
/*14*/     return x;
       }
}
