/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 6.6 (Simple Setup rejection, p. 141)
 *
 * this file: algo06_06.c
 *
 * For a demo how to use this algorithm see the file "demo_algo06_06.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

static double fm,m,vm,ul,ur,a,al,ar, (*pdfh)(double);

int setup06_06( double (*pdf)(double), double Af, double mi, double Fm)
/*
 * Algorithm 6.6 (Simple Setup rejection, p. 141)
 *
 * pdf ... function pointer to the quasi density
 * Af ... area below the quasi density
 * mi ... mode of the density
 * Fm ... CDF value at the mode m; if unknown -1. must be used
 */
{     double um;
      pdfh = pdf;
      m = mi;

/*1*/ fm = (*pdf)(m);
      vm = sqrt(fm);
      um = Af/vm;
/*2*/ if(Fm>=0.){/*the case cdf(m) known*/
/*3*/	a=2.*Af;
/*4*/	al=Fm*a/2.;
	ar=a/2+al;
/*5*/	ul=-Fm*um;
	ur=um+ul;
      }
/*6*/ else{
/*7*/	a=4*Af;
/*8*/	al=a/4;
	ar=3*a/4;
/*9*/	ul=-um;
	ur=um;
      }
      printf("Setup06_06:\n ar %f al %f a %f\n\n",ar,al,a);
      return 1;
} 

double generator06_06()
/* is called after the setup to generate random variates */
{ double u,v,x,y;

/*10*/ while(1){
/*11*/   v = a*uniform();
/*12*/   if(v<al){
/*13*/     x=-ul*ul/v+m;
/*14*/     y=v*v/(ul*ul);
         }
/*15*/   else if(v<=ar){
/*16*/     x=ul/vm+(v-al)/(vm*vm)+m;
/*17*/     y=fm;
         }
/*18*/   else{ 
/*19*/     x=ur*ur/(vm*ur-(v-ar))+m;
/*20*/     y=(a-v)*(a-v)/(ur*ur);
         }
/*21*/   u = uniform();
/*22*/   if(u*y<=(*pdfh)(x))
/*23*/     return x;
       }
}
