/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Algorithm 10.2 (Indexed-Search-2-Tails), p. 220
 *
 * this file: algo10_02.c
 *
 * For a demo how to use this algorithm see the file "demo_algo10_02.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

  static int *gt=NULL,Ch,il,ir;
  static double *pc=NULL, (*pmfh)(int);


int setup10_02(double (*pmf)(int), int ili, int iri, int C)
/*
 * Algorithm 10.2 (Indexed-Search-2-Tails), p. 220
 * generates discrete variates
 *
 * pmf... function pointer to probability (mass) function for k >= 0
 * ili iri ... (possibly short) interval that contains all modes of the pmf
 * C ... length of guide table
 *
 */
{     double pm,pp,*p,s;
      int k,j,l;
      if(pc!=NULL){
	free(pc);
	free(gt);
      }
      il = ili;
      ir = iri;
      l = C;
      if(l<ir+1) l=ir+1;
      pc = malloc(l*sizeof(double));
      p = malloc(l*sizeof(double));
      gt = malloc(C*sizeof(int));
      Ch = C;
      pmfh = pmf;
/*1*/ s=0;
      for(k=il;k<=ir;k++){
	p[k]=(*pmf)(k);
	s+=p[k];
      }
/*2*/ pm = (*pmf)(il-1);
      pp = (*pmf)(ir+1);
/*3*/ while(s<1.-1./C){
/*4*/	if(pm<pp){
/*5*/	  s = s+pp;
	  ir = ir+1;
	  if(l<ir+1){ 
	    l*=2;
	    pc = realloc(pc,l*sizeof(double));
	    p = realloc(p,l*sizeof(double));
	  }
	  p[ir] = pp;
/*6*/	  pp = (*pmf)(ir+1);	
	}
/*7*/	else{
/*8*/	  s = s+pm;
	  il = il-1;
	  p[il] = pm;
/*9*/	  pm = (*pmf)(il-1);
	}
/*10*/	s=0.;
	for(k=il;k<=ir;k++){
	  s += p[k];
	  pc[k] = s;
	}
      }
      free(p);
      pc = realloc(pc,(ir+1)*sizeof(double));
/*11*/gt[0]=il;  
      k=il;
/*12*/for(j=1; j<C; j++){
/*13*/	while(j/(double)C > pc[k]) 
/*14*/    k++;
/*15*/	gt[j] = k;
      }
      printf("setup10_02:\n   probabilities stored in the interval il %d, ir%d\n\n",il,ir);
      return 1;
}


int generator10_02()
/* is called after the setup to generate random variates */
{      double u,s;
       int x,j;
/*16*/ u = uniform();
/*17*/ j = (int)(u*Ch);
/*18*/ x = gt[j];
/*19*/ if(j< Ch-1){
/*20*/	 while(u > pc[x]) 
/*21*/     x++;
/*22*/	 return x;
       }
/*23*/ else{
/*24*/	 s = pc[x];
/*25*/	 if(u<s) 
/*26*/     return x;
/*27*/	 while(1){
/*28*/	   x++;
/*29*/	   s =s + (*pmfh)(x);
/*30*/	   if(u<s) 
/*31*/       return x;
/*32*/	   s =s + (*pmfh)(il+ir-x);
/*33*/	   if(u<s) 
/*34*/       return il+ir-x;
	 }
       }
}

int free10_02()
/* to free the allocated memory*/
{
    free(pc);
    pc = NULL;
    free(gt);
    gt = NULL;
    return 1;
}



