/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 11.3 (Multinormal-Cholesky, p. 250)
 *
 * this file: algo11_03.c
 *
 * For a demo how to use this algorithm see the file "demo_algo11_03.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"
#include "algo.h"
#include "statutil.h"


static int D;
static double *L=NULL, *eps=NULL;

int setup11_03(int d, double *S)
/*
 * Algorithm 11.3 (Multinormal-Cholesky, p. 250)
 *
 * d ... dimension (length) of the generated vectors
 * S ... pointer to the array that holds the consecutive columns 
 *       of the variance-covariance matrix
 */
{     

      D = d;
      if(L!=NULL){
        free(L);
	free(eps);
      }
      L = malloc((D+1)*(D+1)*sizeof(double));  
      eps = malloc((D+1)*sizeof(double));  
/*1*/ if(!algo11_04(d,S,L)){
	printf("\nError setup11_03:\n"); 
	printf("the variance-covariance matrix seems to be not positive definite\n");
	return 0;
      }
      printf("\nsetup11_03 was successful. The calculated Cholesky factor is:\n");
      printmatrix(D,L);
      return 1;
}


#define I(i,j) ((i)*(D+1)+(j))

void generator11_03(double x[])
/* 
 * is called after the setup to generate a random vector 
 * x ... is the array to hold teh generated vector     
 */
{     int i,j;
      double sum;

/*2*/ for(i=1;i<=D;i++) eps[i]=generator02_11();
/*3*/ for(i=1;i<=D;i++){
        sum=0.;
	for(j=1;j<=i;j++) sum+=L[I(i,j)]*eps[j];
	x[i] = sum;
      }
}
#undef I

int free11_03()
/* frees the allocated memory */
{
  free(L);
  L = NULL;
  free(eps);
  eps = NULL;
  return 1;
}

