/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Example 2.14, p. 28, 
 * "1/x density" on (1,3) by composition-rejection with constant hat and squeeze 
 *
 * this file: example02_14.c
 *
 * to compile this file you can use the command:
 *
 * cc -o example02_14 example02_14.c uniform.c statutil.c -lm
 *
 */


#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "statutil.h"
#include "uniform.h"

/**********************************************************************/

double example02_14(void)
/* 
 * generates random variates from the "1/x density" on (1,3) by 
 * composition-rejection with constant hat and squeeze 
 */
{ double b[3],w1,w2,u,x,v,y,h[3],s[3];
  int j;

/*1*/  b[0]=1.; 
       b[1]=sqrt(3.); 
       b[2]=3.; 
       w1=sqrt(3.)-1.; 
       w2=(3.-sqrt(3.))/sqrt(3.);
/*2*/  h[1]=1.; 
       h[2]=1./sqrt(3.); 
       s[1]=1./sqrt(3.); 
       s[2]=1./3.;
/*3*/  while(1){ 
/*4*/    u = (w1+w2)*uniform();
/*5*/    if(u<w1)
/*6*/      j=1;
/*7*/    else 
/*8*/      j=2;
/*9*/    u = uniform();
/*10*/   x = b[j-1]+u*(b[j]-b[j-1]);
/*11*/   v = uniform();
/*12*/   y = v*h[j];
/*13*/   if(y<=s[j]) 
/*14*/     return x;
/*15*/   if(y<=1/x) 
/*16*/     return x;
       }
}

/*****************************************************************/


#define CLAS 500

static long int obs[CLAS*CLAS];

int main()
{ int i;
 double x;

 setseed(13324885); /*set seed for uniform generator*/

 printf("Example 2.14:\n\nfirst we display ten generated numbers\n");

 for(i=0;i<10;i++){
   x = example02_14();
   printf("x %f\n",x);
 }
 
 printf("\nNow we make a chi-square test with sample size %d\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<CLAS*CLAS;i++){
   x = example02_14();
   obs[(int)(cdf_oox1_3(x)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}






