/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Example 11.1, p. 247
 * "Uniform distribution over a triangle"
 *
 * this file: example11_01.c
 *
 * to compile this file you can use the command:
 *
 * cc -o example11_01 example11_01.c uniform.c statutil.c -lm
 *
 */


#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "statutil.h"
#include "uniform.h"

/**********************************************************************/

void example11_01(double x[2])
/* 
 * generates random pairs uniformly distributed over the triangle 
 * with vertices (0,0), (2,0), (2,1)
 * there is no input necessary
 * The array x is used to hold the generated pair
 */
{     double u,v;

/*1*/ u = uniform();
/*2*/ x[0] = 2.*sqrt(u);
/*3*/ v = uniform();
/*4*/ x[1] = v*x[0]/2;
}

/*****************************************************************/


#define CLAS 500

static long int obs[CLAS*CLAS];

int main()
{ int i;
 double x[2];

 setseed(13324885); /*set seed for uniform generator*/

 printf("Example 11.1\nfirst we display ten generated pairs\n");

 for(i=0;i<10;i++){
   example11_01(x);
   printf("( %f | %f )\n",x[0],x[1]);
 }
 
 printf("\nNow we test if 2.x1/x0 is uniform.\n\n");
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<CLAS*CLAS;i++){
   example11_01(x);
   obs[(int)((2.*x[1]/x[0])*CLAS)]++;
 } 
 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");

 return 0;
}






